%% Load the data and display transmission sinogram
% The data files (nut.mat, dino.mat, and plusplus.mat) contain the following:
%
%  T      transmission sinogram (each column is a normalized projection)
%  R      distance from source to center of rotation (in cm)
%  dw     width of detector (in cm)
%  sd     distance from source to detector (in cm)
%  theta  vector of projection angles (in degrees)
%

clear all

% Uncomment one of the three lines below to load a data set
%load nut.mat        % Center slice of cone beam projections of nut
%load dino.mat       % Center slice of cone beam projections of dino
%load plusplus.mat   % Center slice of cone beam projections of plusplus
%load match.mat      % Center slice of cone beam projections of match
load wizard.mat     % Center slice of cone beam projections of wizard

if exist('fanlineartomo') ~= 2
    error('The script requires the function ''fanlineartomo'' from the MATLAB package ''AIR Tools II''.')
end

%%
idx = 1:8:length(theta);    % indices of angles to use for reconstruction
[ndet, nproj] = size(T);    % no. det. pixels, no. projections
dew = dw/ndet;              % width of detector element
dcnt = linspace(-dw/2+dew/2,dw/2-dew/2,ndet); % centers of det. elements

if length(idx) ~= length(theta)
    warning('Using a subset of the available projections.')
end

% Plot pre-log. sinogram
figure(1)
colormap(gray)
imagesc(theta,dcnt,T,[0,1.1]), colorbar
title('Pre-logarithm sinogram')
ylabel('Position [cm]')
xlabel('Projection angle [deg.]')
set(gca,'fontsize',14)

%% Model setup
% Set up the forward model using AIR Tools II and compute sinogram

N = 256;              % reconstruction resolution
grid_size = dw*R/sd;  % width of reconstruction grid in cm

A = fanlineartomo(N,theta(idx),ndet,R/grid_size,dw/grid_size,sd/grid_size);
A = A*(grid_size/N);
b = -log(double(reshape(T(:,idx),[],1)));

%% Reconstruction
% Compute simple least-squares reconstruction using PGD with a constant 
% step-size and a fixed number of iterations

maxiters = 100;     % number of iterations
rho = 1.5;          % step size multiplier (0 < rho < 2)

L = normest(A)^2;   % compute/estimate Lipschitz constant
xl = zeros(N*N,1);  % initial x
rk = -b;            % initial residual
objval = zeros(maxiters,1);
for k = 1:maxiters
    xl = max(0,xl - rho/L*(A'*rk));
    rk = A*xl-b;
    objval(k) = 0.5*norm(rk)^2;
end

%% Data visualization
% Fig. 2: Reconstruction, "optimality conditions", 
% model sinogram, and residuals
%
% Fig. 3: Objective value as a function of iterations

figure(2)
colormap(gray)
subplot(2,2,1)
xt = linspace(-grid_size/2+grid_size/(2*N),grid_size/2-grid_size/(2*N),N);
imagesc(xt,xt,reshape(xl,N,N)), axis image, colorbar
xlabel('[cm]')
ylabel('[cm]')
title('Reconstruction (x)')
set(gca,'fontsize',14)

subplot(2,2,2)
g = A'*rk;
imagesc(xt,xt,reshape((xl>0).*g + g.*(xl==0).*(g<0),N,N)), axis image, colorbar
colormap(gray)
xlabel('[cm]')
ylabel('[cm]')
title('"Optimality"')
set(gca,'fontsize',14)

subplot(2,2,3)
imagesc(theta(idx),dcnt,reshape(A*xl,ndet,length(idx))), colorbar
colormap(gray)
ylabel('Position [cm]')
xlabel('Projection angle [deg.]')
title('Model sinogram (Ax)')
set(gca,'fontsize',14)

subplot(2,2,4)
imagesc(theta(idx),dcnt,reshape(rk,ndet,[])), colorbar
colormap(gray)
ylabel('Position [cm]')
xlabel('Projection angle [deg.]')
title('Residual (Ax-b)')
set(gca,'fontsize',14)

figure(3)
semilogy(0:maxiters,[0.5*norm(b)^2;objval],'k.-')
grid on
xlabel('Number of iterations')
ylabel('Objective value')
set(gca,'fontsize',14)
